@@ -6,36 +6,36 @@ import time
 import datetime
 import pytz
 from dateutil import relativedelta
-
 import aiohttp
 import asyncio
 
+aiohttpResponse = aiohttp.client_reqrep.ClientResponse
 
 
 class YFClient:
 
-    snake_pattern = re.compile(r'(?<!^)(?=[A-Z])')# {{{
-    BASE_URL = 'https://query1.finance.yahoo.com'
-    SCREENER_URL = BASE_URL + '/v1/finance/screener'
-    QUOTE_SUMMARY_URL = BASE_URL + '/v10/finance/quoteSummary/'
-    TRENDING_URL = BASE_URL + '/v1/finance/trending/'
-    FIELD_URL = BASE_URL + '/v1/finance/screener/instrument/{asset_class}/fields'
-    QUOTE_URL = BASE_URL + '/v7/finance/quote' #params symbol
-    RECCO_URL = BASE_URL + '/v6/finance/recommendationsbysymbol/{symbol}'
-    PEER_ESG_URL = BASE_URL + '/v1/finance/esgPeerScores' #params symbol
-    SEARCH_URL = BASE_URL + '/v1/finance/search' # params q, quoteCount, newsCount, enableFuzzyQuery,
-    HISTORY_URL = BASE_URL + '/v8/finance/chart/{symbol}'
-    TIME_URL = BASE_URL + '/v6/finance/markettime' # params region, lang=en-US
 
-    DUMMY_URL = 'https://finance.yahoo.com/quote/AAPL'
+    _BASE_URL: str = 'https://query1.finance.yahoo.com'# {{{
+    _SCREENER_URL: str = _BASE_URL + '/v1/finance/screener'
+    _QUOTE_SUMMARY_URL: str = _BASE_URL + '/v10/finance/quoteSummary/'
+    _TRENDING_URL: str = _BASE_URL + '/v1/finance/trending/'
+    _FIELD_URL: str = _BASE_URL + '/v1/finance/screener/instrument/{asset_class}/fields'
+    _QUOTE_URL: str = _BASE_URL + '/v7/finance/quote' #params symbol
+    _RECCO_URL: str = _BASE_URL + '/v6/finance/recommendationsbysymbol/{symbol}'
+    _PEER_ESG_URL: str = _BASE_URL + '/v1/finance/esgPeerScores' #params symbol
+    _SEARCH_URL: str = _BASE_URL + '/v1/finance/search' # params q, quoteCount, newsCount, enableFuzzyQuery,
+    _HISTORY_URL: str = _BASE_URL + '/v8/finance/chart/{symbol}'
+    _TIME_URL: str = _BASE_URL + '/v6/finance/markettime' # params region, lang=en-US
 
-    HEADERS = {'Content-Type': 'application/json',
+    _DUMMY_URL: str = 'https://finance.yahoo.com/quote/AAPL'
+
+    _HEADERS: dict = {'Content-Type': 'application/json',
             'Origin': 'https://finance.yahoo.com',
             'Accept-Language': 'en-gb',
             'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/14.1 Safari/605.1.15',
             }
 
-    QUOTE_SUMMARY_MODULES = ['assetProfile', 'secFilings', 
+    _QUOTE_SUMMARY_MODULES: list = ['assetProfile', 'secFilings', 
             'incomeStatementHistory', 'cashflowStatementHistory', 'balanceSheetHistory',
             'incomeStatementHistoryQuarterly', 'cashflowStatementHistoryQuarterly', 'balanceSheetHistoryQuarterly',
             'earningsHistory', 'earningsTrend', 'industryTrend', 'indexTrend', 'sectorTrend',
@@ -43,148 +43,114 @@ class YFClient:
             'financialData', 'recommendationTrend', 'upgradeDowngradeHistory', 'earnings', 'details',
             'summaryDetail', 'price', 'pageViews', 'financialsTemplate', 'components',
             'institutionOwnership', 'fundOwnership', 'majorDirectHolders', 'insiderTransactions',
-            'insiderHolders', 'majorHoldersBreakdown', 'netSharePurchaseActivity', 'fundPerformance']
-
-    QUOTE_COLUMNS = ['symbol', 'uuid', 'underlyingExchangeSymbol', 'messageBoardId', 'longName',
-            'shortName', 'marketCap', 'underlyingSymbol', 'headSymbolAsString', 'isin',
-            'regularMarketPrice', 'regularMarketChange', 'regularMarketChangePercent', 
-            'regularMarketVolume', 'regularMarketOpen', 'fiftyTwoWeekLow', 'fiftyTwoWeekHigh',
-            'toCurrency', 'fromCurrency', 'toExchange', 'fromExchange', 'bid', 'ask', 'currency',
-            'bidSize', 'askSize', 'averageDailyVolume3Month', 'averageDailyVolume10Day',
-            'marketState', 'beta', 'preMarketTime', 'preMarketChange', 'preMarketChangePercent',
-            'preMarketVolume', 'preMarketOpen', 'currentTradingPeriod', 'tradingPeriods',
-            'preMarketPrice', 'preMarketDayHigh', 'preMarketDayLow', 'preMarketPreviousClose',
-            'regularMarketPreviousClose', 'postMarketPreviousClose', 'postMarketVolume',
-            'postMarketTime', 'regularMarketClose', 'last', 'trade',
-            'postMarketChangePercent', 'postMarketPrice', 'postMarketChange',
-            'language', 'region', 'quoteType', 'quoteSourceName', 'triggerable',
-            'sourceInterval', 'exchangeDataDelayedBy', 'tradeable', 'firstTradeDateMilliseconds',
-            'priceHint', 'regularMarketTime', 'regularMarketDayHigh', 'regularMarketDayRange',
-            'regularMarketDayLow', 'fullExchangeName', 'fiftyTwoWeekLowChange', 'fiftyTwoWeekLowChangePercent',
-            'fiftyTwoWeekRange', 'fiftyTwoWeekHighChange', 'fiftyTwoWeekHighChangePercent',
-            'exchange', 'exchangeTimezoneName', 'exchangeTimezoneShortName', 'gmtOffSetMilliseconds',
-            'market', 'esgPopulated', 'sharesOutstanding'] #not used rn
-
-
-    SCREENER_COLS = ['symbol', 'exchange', 'currency', 'fullExchangeName', 'quoteSourceName', 
-            'shortName', 'longName', 'displayName', 'quoteType', 'firstTradeDateMilliseconds',
-            'priceHint', 'market', 'messageBoardId', 'financialCurrency', 'sourceInterval', 'exchangeDataDelayedBy',
-            'exchangeTimezoneName', 'exchangeTimezoneShortName', 'gmtOffSetMilliseconds', 'prevName', 'nameChangeDate']
-
-    PRICE_HISTORY_COLS = ['symbol', 'date', 'close', 'volume', 'low', 'high',
-                'adjclose', 'ratio', 'adjopen', 'adjhigh', 'adjlow', 'interval', 'date_local',
-                'trading_period']
-
+            'insiderHolders', 'majorHoldersBreakdown', 'netSharePurchaseActivity', 'fundPerformance',
+            'topHoldings', 'fundProfile', 'quoteType']
 # }}}
 
-    def __init__(self):# {{{
-        pass# }}}
-
-    async def connect(self, *args, **kwargs):# {{{
-
-        self.proxy = None
-        if 'proxy' in kwargs:
-            self.proxy = kwargs['proxy']
-
-        self.crumb, self.cookies = self._get_crumb()
-
-        connector = aiohttp.TCPConnector(limit=10)
-        self.session = aiohttp.ClientSession(connector=connector,
-                headers=self.HEADERS, cookies=self.cookies)
+    async def connect(self, limit: int|None = 2) -> None:# {{{
+        """
+        Parameters
+        ----------
+        limit:
+            The maximum number of simultaneously open connections. If none
+            then there is no maximum limit
+        """
+        self._limit: int|None = limit
 
+        self._crumb: str
+        self._cookies: requests.cookies.RequestsCookieJar
+        self._crumb, self._cookies = self._get_crumb()
 
+        connector: aiohttp.TCPConnector = aiohttp.TCPConnector(limit=self._limit)
+        self.session: aiohttp.ClientSession = aiohttp.ClientSession(connector=connector,
+                headers=self._HEADERS, cookies=self._cookies)
         # }}}
 
-    async def get_new_crumb(self):# {{{
-        self.crumb, self.cookies = self._get_crumb()
-        await self.session.close()
-        connector = aiohttp.TCPConnector(limit=10)
-        self.session = aiohttp.ClientSession(connector=connector,
-                headers=self.HEADERS, cookies=self.cookies)
+    async def get_new_crumb(self) -> None:# {{{
+        await self.disconnect()
+        await self.connect()
         # }}}
 
-    def _get_crumb(self):# {{{
+    def _get_crumb(self) -> tuple[str, requests.cookies.RequestsCookieJar]:# {{{
         """method to retrieve the crumb that yahoo uses for api authentication"""
 
         # tmp = requests.get(self.DUMMY_URL, headers=self.HEADERS, proxies=self.proxy)
-        tmp = requests.get(self.DUMMY_URL, headers=self.HEADERS, proxies={'https': self.proxy})
-        crumb = re.findall('"CrumbStore":{"crumb":"(.+?)"}', tmp.text)[0]
-        cookies = tmp.cookies
+        tmp : requests.models.Response = requests.get(self._DUMMY_URL, headers=self._HEADERS)
+        crumb: str = re.findall('"CrumbStore":{"crumb":"(.+?)"}', tmp.text)[0]
+        cookies: requests.cookies.RequestsCookieJar = tmp.cookies
 
         return crumb, cookies# }}}
 
-    async def _make_request(self, method, url, *args, **kwargs):# {{{
+    async def _make_request(self, method: str, url: str, **kwargs) -> aiohttpResponse:# {{{
         """function that makes all the requests async"""
 
-        params = {'crumb': self.crumb}
+        crum: dict = {'crumb': self._crumb}
 
         if 'params' in kwargs:
-            params.update(kwargs['params'])
-            del kwargs['params']
+            kwargs['params'].update(crum)
+        else:
+            kwargs['params'] = crum
 
-        resp = await self.session.request(method, url, params=params, proxy=self.proxy, *args, **kwargs)
-        return resp# }}}
+        return await self.session.request(method, url, **kwargs) # }}}
 
-    async def disconnect(self):# {{{
+    async def disconnect(self) -> None:# {{{
+        """close the client session"""
         await self.session.close()# }}}
 
-    async def _get_quote(self, symbols):# {{{
+    async def _get_quote(self, symbols: list, fields: list) -> list:# {{{
 
-        url = self.QUOTE_URL
+        url: str = self._QUOTE_URL
+        params: dict = {'symbols': ','.join(symbols)}
+        if fields:
+            params.update({'fields': ','.join(fields)})
 
-        resp = await self._make_request('get', url, params={'symbols': ','.join(symbols), 'fields': ','.join(self.QUOTE_COLUMNS)})
+        resp: aiohttpResponse = await self._make_request('get', url, params=params)
 
-        resp = await resp.json()
+        resp: dict = await resp.json()
         if resp['quoteResponse']['error']:
             raise ValueError(f'{resp["quoteResponse"]["error"]}')
 
-        res = resp['quoteResponse']['result']
-        if res:
-            return res
-        else:
-            return [{'symbol': x} for x in symbols]
+        resp: list = resp['quoteResponse']['result']
+        return resp
         # }}}
 
-    async def get_quote(self, symbols, columns=None):# {{{
+    async def get_quote(self, symbols: str|list, fields: str|list =None) -> dict:# {{{
+        """
+        Parameters
+        ----------
+        symbols:
+            The yahoo finance symbols to get data for
+        fields:
+            The yahoo finance fields to get data for. If not specified, it
+            will return all available fields for the given instrument
 
-        if isinstance(columns, str):
-            columns = [columns]
+        Returns
+        -------
+        dict:
+            A dictionary where the keys are the symbols, and the values are a dictionary
+            of key value pairs for each field.
+            Note: Even when a subset of fields is specified - the yahoo finance
+            API will always return certain fields (i.e. timestamp, symbol)
+        """
 
-        if isinstance(symbols, str):
-            if ',' in symbols:
-                symbols = symbols.split(',')
-            else:
-                symbols = [symbols]
+        fields: list = [fields] if isinstance(fields, str) else fields
+        symbols: list = [symbols] if isinstance(symbols, str) else symbols
 
 
-        chunks = [symbols[x:x+50] for x in range(0, len(symbols), 50)]
-        tasks = []
-        for chunk in chunks:
-            tasks.append(self._get_quote(chunk))
+        chunks: list = [symbols[x:x+50] for x in range(0, len(symbols), 50)]
+        tasks: list = [self._get_quote(chunk, fields) for chunk in chunks]
 
-        res = await asyncio.gather(*tasks)
+        res: list = await asyncio.gather(*tasks)
 
-        empty_df = pd.DataFrame(columns=self.QUOTE_COLUMNS)
+        res: list = [inner for outer in res for inner in outer]
 
-        df = pd.concat([empty_df] + [pd.DataFrame(x) for x in res])
+        fres: dict = {}
+        for r in res:
+            sym = r.pop('symbol')
+            fres[sym] = r
 
-        if columns:
-            if 'symbol' not in columns:
-                columns += ['symbol']
-            df = df[columns]
-
-        for c in df.columns:
-            if c not in self.QUOTE_COLUMNS:
-                print(f'column {c} found in quote df but not in self.QUOTE_COLUMNS')
-
-        for c in self.QUOTE_COLUMNS:
-            if c not in df.columns:
-                df[c] = None
-
-        return df
-        # df.columns = self.camel_to_snake(list(df.columns))
-
-        # return df# }}}
+        return fres
+        # }}}
 
     async def _price_history(self, symbol, interval=None, start=None, end=None, adjust=True,# {{{
             prepost=False):
@@ -194,7 +160,7 @@ class YFClient:
         assert isinstance(end, datetime.datetime); assert end.tzinfo
         assert isinstance(start, datetime.datetime); assert start.tzinfo
 
-        url = self.HISTORY_URL.format(symbol=symbol)
+        url = self._HISTORY_URL.format(symbol=symbol)
 
 
         params = {'includeAdjustedClose': str(adjust).lower(), 'events': 'div,splits,capitalGain',
@@ -214,7 +180,7 @@ class YFClient:
         if 'finance' in resp.keys():
             print('finance error')
             print(f'{resp["finance"]["error"]}')
-            return
+            raise ValueError(resp['chart']['error'])
         if resp['chart']['error']:
             if resp['chart']['error']['description'][0:35] == 'Data doesn\'t exist for startDate = ':
                 print("no results found for period")
@@ -222,6 +188,7 @@ class YFClient:
             else:
                 print('chart error')
                 print(f'{resp["chart"]["error"]}')
+                raise ValueError(resp['chart']['error'])
                 return
 
         if not resp['chart']['result'][0]['indicators']['quote'][0]:
@@ -295,7 +262,10 @@ class YFClient:
         """
 
         if isinstance(symbols, str):
-            symbols = [symbols]
+            if ',' in symbols:
+                symbols = symbols.split(',')
+            else:
+                symbols = [symbols]
 
         tasks = []
         for t in symbols:
@@ -315,58 +285,43 @@ class YFClient:
             return pd.DataFrame(columns=self.PRICE_HISTORY_COLS)
         # }}}
 
-    def _build_screener_payload(self,# {{{
-            sort_field='intradaymarketcap', sort_type='DESC', quote_type='EQUITY',
-            mcap_filter=100_000_000, region='us'):
+    def _build_screener_payload(self, sort_field: str = 'intradaymarketcap', # {{{
+            sort_type: str = 'DESC', quote_type: str = 'EQUITY',
+            mcap_filter: int = 100_000_000, region: str|list[str] = 'us',
+            exchange: str|None = None, currency: str|None = None) -> dict:
 
-        if isinstance(region, str):
-            region = [region]
+        region: list = [region] if isinstance(region, str) else region
 
 
-        payload = {
+        payload: dict = {
                 'size': 100,
                 'offset': 0,
                 'sortField': sort_field,
                 'sortType': sort_type,
                 'quoteType': quote_type,
                 'topOperator': 'AND',
-                # 'includeFields': ['ticker','isin', 'companyshortname', 'sector', 'neutral_count'],
                 'query': {
                     'operator': 'AND',
                     'operands': [
-                        # {
-                        #     'operator': 'gt',
-                        #     'operands': ['lastclosemarketcap.lasttwelvemonths', mcap_filter]
-                        # },
-                 #        {
-                 #            'operator': 'EQ',
-                 #            'operands': ['isin', 'US0378331005']
-                 #        },
                         ],
                 'userId': '',
                 'userIdType': 'guid'
                 } }
 
-        to_add = payload['query']['operands']
+        to_add: list = payload['query']['operands']
 
         if mcap_filter:
             to_add.append({'operator': 'AND', 'operands': 
                 [{'operator': 'gt', 'operands': ['lastclosemarketcap.lasttwelvemonths', mcap_filter]}]})
 
+        if exchange:
+            to_add.append({ "operator": "eq", "operands": [ "exchange", exchange] })
+
+        if currency:
+            to_add.append({ "operator": "eq", "operands": [ "currency", currency ] })
+
         if region:
-           #  reg_query = {
-           #          "operator": "or",
-           #          "operands": [
-           #              {
-           #                  "operator": "EQ",
-           #                  "operands": [
-           #                      "region",
-           #                      region
-           #                      ]
-           #                  }
-           #              ]
-           #          }
-            reg_query = {
+            reg_query: dict = {
                     "operator": "or",
                     "operands": []}
 
@@ -378,21 +333,19 @@ class YFClient:
             to_add.append(reg_query)
         return payload# }}}
 
-    async def _send_screener_request(self, payload):# {{{
+    async def _send_screener_request(self, payload: dict) -> dict:# {{{
 
+        # print(f"sending: {payload}")
         print('sending')
-        resp = await self._make_request('post', self.SCREENER_URL, json=payload)
+        resp: aiohttpResponse = await self._make_request('post', self._SCREENER_URL, json=payload)
 
-        # data = resp.json()['finance']['result'][0]['quotes']
         try:
-            result = await resp.json()
+            result: dict = await resp.json()
         except:
-            cntnt = await resp.text()
-            print(payload)
+            cntnt: str = await resp.text()
             raise ValueError(f'couldnt convert to json {cntnt}')
 
-        if result['finance']['error']:
-            print(payload)
+        if result['finance'].get('error', None):
             if result['finance']['error']['description'] == 'Invalid Crumb':
                 await self.get_new_crumb()
                 print('getting new crumb')
@@ -400,11 +353,13 @@ class YFClient:
             else:
                 raise ValueError(f'{result["finance"]["error"]}')
 
-        # return pd.DataFrame(data)
-        return result['finance']['result'][0]
-        # }}}
+        if not result['finance'].get('result'):
+            raise ValueError(f'undefined error {result}')
+        fres: dict = result['finance']['result'][0]
+        # print(fres['start'], fres['count'], fres['total'], len(fres['quotes']))
+        return fres # }}}
 
-    async def _iter_screener_requests(self, payload, max_results):# {{{
+    async def _iter_screener_requests(self, payload: dict, max_results: int|None) -> list:# {{{
 
         if not max_results:
             max_results = 10000
@@ -412,94 +367,66 @@ class YFClient:
         payload['size'] = 100
         payload['offset'] = 0
 
-        total = 0
+        total: int = 0
 
 
-        data = []
+        data: list = []
 
 
         while True:
-            result = await self._send_screener_request(payload)
+            result: dict = await self._send_screener_request(payload)
             data.extend(result['quotes'])
 
-            if len(result['quotes']) < payload['size']:
+
+            if result['count'] == 0:
+                print(f'nothing returned now so breaking')
+                print(f'len of data: {len(data)}; stated total: {result["total"]}')
                 break
+            # this doesnt currently work
+            #if result['total'] <= len(data):
+            #    print(f'result["total"] - all results have been returned')
+            #    break
+            # crypto screener some queries have less than this so commenting out. need to see if breaks equity
+            #if len(result['quotes']) < payload['size']:
+            #    print(f'result has less than {payload["size"]} items so breaking')
+            #    break
 
             if len(data) >= max_results:
+                print(f'data is now longer than {max_results} so breaking')
+                break
+            new_offset: int = result['start'] + result['count']
+            if new_offset > 9900:
+                print(f'offset of {new_offset} would be greater than max results so breaking')
                 break
-            new_offset = result['start'] + result['count']
             payload['offset'] = new_offset
 
-        data = data[:max_results]
+        data: list = data[:max_results]
 
         return data# }}}
 
-    async def get_equity_reference(self, region=['us'], max_results=10000, mcap_filter=100_000_000):# {{{
+    async def get_equity_reference(self, region: str|list[str] = ['us'], max_results: int = 10000,# {{{
+            mcap_filter: int = 100_000_000) -> pd.DataFrame:
 
         print(f'getting data for {region}')
-        payload = self._build_screener_payload(region=region, mcap_filter=mcap_filter)
-        res = await self._iter_screener_requests(payload, max_results)
+        payload:dict = self._build_screener_payload(region=region, mcap_filter=mcap_filter)
+        res:dict = await self._iter_screener_requests(payload, max_results)
 
-        data = pd.DataFrame(columns=self.SCREENER_COLS)
-        data = data.append(pd.DataFrame(res))
+        return pd.DataFrame(res)# }}}
 
+    async def get_crypto_reference(self, currency: str = 'USD', max_results: int = 100) -> pd.DataFrame:# {{{
+        payload:dict = self._build_screener_payload(region=None, mcap_filter=None, quote_type='CRYPTOCURRENCY',
+                exchange='CCC', currency=currency)
+        res:dict = await self._iter_screener_requests(payload, max_results)
+        return pd.DataFrame(res)# }}}
 
-        # cols = []
-        # for c in self.SCREENER_COLS:
-        #     if c in data.columns:
-        #         cols.append(c)
-        #     else:
-        #         pass
-        #         # print(f'column {c} not found in ref data')
+    async def get_etf_reference(self, region:str|list[str]=['us'], max_results:int=100):# {{{
+        payload:dict = self._build_screener_payload(region=region, quote_type='ETF', sort_field='fundnetassets', mcap_filter=None)
+        res:dict = await self._iter_screener_requests(payload, max_results)
+        return pd.DataFrame(res)# }}}
 
-        # for c in data.columns:
-        #     if c not in self.SCREENER_COLS:
-        #         pass
-        #         # print(f'column {c} not found in defined screen columns')
+    async def _get_quote_summary(self, ticker:str, modules:None|list[str]=None):# {{{
 
-        print(f'GOT data')
-        return data[self.SCREENER_COLS]# }}}
-
-    async def get_all_equity_reference(self):# {{{
-
-        regions = await self._get_region_reference()
-
-        result = pd.DataFrame()
-
-        tasks = []
-
-        # regions = regions[0:20]
-
-        for reg in regions:
-            # filtering us more strictly than other regions
-            # this is partly because of better mcap data coverage
-            # and partly because there are far more securities ther
-            if reg['code'] == 'us':
-                mcap_filter = 100_000_000
-            else:
-                mcap_filter = None
-            tmp = self.get_equity_reference(region=reg['code'], mcap_filter=mcap_filter)
-            tasks.append(tmp)
-
-
-        res = await asyncio.gather(*tasks)
-
-        for reg, r in zip(regions, res):
-            r['region_code'] = reg['code']
-            r['region_name'] = reg['name']
-
-        df = pd.concat([pd.DataFrame(x) for x in res])
-
-        return df# }}}
-
-    async def _get_quote_summary(self, ticker, modules=None):# {{{
-
-        if isinstance(modules, str):
-            modules = [modules]
-        if not modules:
-            modules = self.QUOTE_SUMMARY_MODULES
-
-        url = self.QUOTE_SUMMARY_URL + ticker
+        url = self._QUOTE_SUMMARY_URL + ticker
         modules = ','.join(modules)
         params = {'modules': modules, 'format': 'false'}
 
@@ -514,50 +441,76 @@ class YFClient:
             print(res_js['quoteSummary']['error'])
             return {} # }}}
 
-    async def get_quote_summary(self, symbols, modules=None):# {{{
+    async def get_quote_summary(self, symbols:str|list[str], modules:None|str|list[str]=None):# {{{
 
-        if isinstance(symbols, str):
-            symbols = [symbols]
+        if not modules:
+            modules = self._QUOTE_SUMMARY_MODULES
+
+        symbols = [symbols] if isinstance(symbols, str) else symbols
+        modules = [modules] if isinstance(modules, str) else modules
 
 
-        tasks = []
-
-        for symb in symbols:
-            tasks.append(self._get_quote_summary(symb, modules=modules))
+        tasks = [self._get_quote_summary(symb, modules=modules) for symb in symbols]
 
         res = await asyncio.gather(*tasks)
 
 
         fres = {}
         for symb, qs in zip(symbols, res):
-            fres[symb] = qs
+            if qs:
+                fres[symb] = qs
 
         return fres# }}}
 
-    async def _get_trending(self, region='us', count=5):# {{{
+    async def _get_trending(self, region:str='us', count:int=5) -> list:# {{{
         print(f'getting {region}')
 
-        url = self.TRENDING_URL + '/' + region.upper()
+        url:str = self._TRENDING_URL + '/' + region.upper()
 
-        req = await self._make_request('get', url, params={'count': count})
+        req:aiohttpResponse = await self._make_request('get', url, params={'count': count})
 
-        resp = await req.json()
+        resp:dict = await req.json()
 
         try:
-            data = resp['finance']['result'][0]['quotes']
-            tmp = []
+            data:list = resp['finance']['result'][0]['quotes']
+            tmp:list = []
             for i in data:
                 tmp.append(i['symbol'])
-            print(f'got {region}')
             return tmp
         except Exception as e:
-            print(resp['finance'])
             return []
         # }}}
 
+    async def get_trending(self, regions:str|list[str]=['us', 'gb'], count:int=5) -> dict:# {{{
+        """
+        Parameters
+        ----------
+        regions:
+            which country to get trending tickers from
+        count:
+            how many tickers to retrieve
+
+        Returns
+        -------
+        dict:
+            Dictionary with keys as regions, and values as lists of symbols
+        """
+
+        regions:list = [regions] if isinstance(regions, str) else regions
+
+        tasks:list = [self._get_trending(region=reg, count=count) for reg in regions]
+
+        res:list = await asyncio.gather(*tasks)
+
+        fres:dict = {}
+        for r, rs in zip(regions, res):
+            if rs:
+                fres[r] = rs
+        return fres# }}}
+
     async def _get_screener_fields(self, asset_class):# {{{
 
-        url = self.FIELD_URL.format(asset_class=asset_class)
+        url = self._FIELD_URL.format(asset_class=asset_class)
 
         req = await self._make_request('get', url)
 
@@ -649,31 +602,12 @@ class YFClient:
                 'post_start': convert_sess('post', 'start'),
                 'post_end': convert_sess('post', 'end')}# }}}
 
-    async def get_trending(self, regions=['us', 'gb'], count=5):# {{{
-
-        if isinstance(regions, str):
-            regions = [regions]
-
-        res = {}
-        tasks = []
-        for reg in regions:
-            tasks.append(self._get_trending(region=reg, count=count))
-
-        res = await asyncio.gather(*tasks)
-
-        fres = {}
-        for r, rs in zip(regions, res):
-            fres[r] = rs
-
-
-        return fres# }}}
-
     async def _get_symbol_recos(self, symbol):# {{{
 
         if isinstance(symbol, str):
             symbol = [symbol]
 
-        res = await self._make_request('get', self.RECCO_URL.format(symbol=','.join(symbol)))
+        res = await self._make_request('get', self._RECCO_URL.format(symbol=','.join(symbol)))
         res = await res.json()
         return res# }}}
 
@@ -713,7 +647,7 @@ class YFClient:
 
     async def _get_markettime(self, region):# {{{
 
-        res = await self._make_request('get', self.TIME_URL,
+        res = await self._make_request('get', self._TIME_URL,
                 params={'lang': 'en-US', 'region': region})
 
         return await res.json()# }}}
@@ -747,7 +681,7 @@ class YFClient:
                 print(f'couldnt find data for {nme}')
                 continue
 
-            tmp = pd.Series({
+            tmp = {
                 'name': nme,
                 'short_yname': t['id'],
                 'yname': t['name'],
@@ -759,22 +693,24 @@ class YFClient:
                 'dst_active': t['timezone'][0]['dst'],
                 'gmtoffset': t['timezone'][0]['gmtoffset'],
                 'timezone_code': t['timezone'][0]['short'],
-                'pytz_name': t['timezone'][0]['$text']})
+                'pytz_name': t['timezone'][0]['$text']}
 
             ffres.append(tmp)
 
-        ffres = pd.concat(ffres, axis=1).T
-        ffres['open'] = pd.to_datetime(ffres['open'])
-        ffres['close'] = pd.to_datetime(ffres['close'])
-        ffres['open'] = ffres.apply(lambda x: x['open'].tz_convert(x['pytz_name']), axis=1)
-        ffres['close'] = ffres.apply(lambda x: x['close'].tz_convert(x['pytz_name']), axis=1)
+        # ffres = pd.concat(ffres, axis=1).T
+        for mkt in ffres:
+            mkt['open'] = pd.to_datetime(mkt['open'])
+            mkt['close'] = pd.to_datetime(mkt['close'])
+            mkt['open'] = mkt['open'].tz_convert(mkt['pytz_name'])
+            mkt['close'] = mkt['close'].tz_convert(mkt['pytz_name'])
+            mkt['dst_active'] = True if mkt['dst_active'].lower() == 'true' else False
 
-        return pd.concat(ffres, axis=1).T# }}}
+        return pd.DataFrame(ffres)# }}}
 
-    async def _get_esg_score(self, symbol, count=5):# {{{
+    async def _get_esg_peer_score(self, symbol, count=5):# {{{
 
 
-        res = await self._make_request('get', self.PEER_ESG_URL, params={'symbol': symbol,
+        res = await self._make_request('get', self._PEER_ESG_URL, params={'symbol': symbol,
             'count': count})
 
         return await res.json()# }}}
@@ -787,7 +723,7 @@ class YFClient:
         tasks = []
 
         for sym in symbols:
-            tasks.append(self._get_esg_score(sym, count))
+            tasks.append(self._get_esg_peer_score(sym, count))
 
         res = await asyncio.gather(*tasks)
 
@@ -827,10 +763,10 @@ class YFClient:
             news_count = count
             quote_count = 0
 
-        params = {'q': query, 'quoteCount': quote_count, 'newsCount': news_count,
+        params = {'q': query, 'quotesCount': quote_count, 'newsCount': news_count,
                 'enableFuzzyQuery': str(fuzzy).lower()}
 
-        res = await self._make_request('get', self.SEARCH_URL, params=params)
+        res = await self._make_request('get', self._SEARCH_URL, params=params)
 
         res = await res.json()
 
@@ -869,13 +805,3 @@ class YFClient:
 
         return pd.DataFrame(res)# }}}
 
-    def camel_to_snake(self, cols):# {{{
-        if isinstance(cols, str):
-            cols = [cols]
-        res = []
-        for c in cols:
-            res.append(self.snake_pattern.sub('_', str(c)).lower())
-        return res# }}}
-
-
-# q['spread'] = ((q['ask'] - q['bid']) / ((q['ask'] + q['bid']) / 2)) * 10000
